#!/usr/bin/perl -w

$_ = <<'END'
<!doctype linuxdoc system>

<article>

<title>Quelques ides pour Combiner des Mtaobjets
<author>Franois-Ren Rideau <tt>rideau@essi.fr</tt>
<date>8 Avril 1997

<abstract>

La programmation rflexive,
les implmentations ouvertes,
les protocoles  mtaobjets,
sont un sujet actif de recherche depuis une vingtaine d'annes;
cependant, malgr leurs succs et leur indubitable potentiel,
ces techniques ``mta-'' rencontrent de grandes difficults,
aussi bien thoriques que pratiques,
quand il s'agit de combiner des mta-objets.
Cette courte tude, partant de la comparaison
de trois articles donns sur ce sujet,
tente d'en extraire les ides essentielles,
d'en souligner les apports positifs,
et d'en relever les points faibles;
en conclusion, nous proposons une approche
pour s'attaquer plus profondment au problme soulev,
mais non rsolu, dans les trois articles.

</abstract>

<!-- toc -->


<sect>Introduction

<sect1>Objets et Mtaobjets
<p>
La thorie des langages informatiques s'attache  donner un sens
 des entits communment appeles ``objets'',
dcrits  l'aide de textes appels ``programmes''.
La plupart des langages dduisent le sens d'un objet
de la structure explicite du programme le dcrivant,
considre dans un contexte absolu et immuable.

   Cependant, pour tout langage,
ce processus est effectu lui-mme par un programme
(interprteur, compilateur, etc), appel ``mtaprogramme'',
ayant lui-mme sa propre structure,
descriptible dans un ``mta''-langage,
pour lequel le langage prcdent devient un ``mta''-objet.

   Dans la pratique, il n'y a pas de rgression infinie
des mtaobjets, car le calcul est finalement excut
par des machines physiques, rgies par les lois de la physique,
au-del desquelles il est vain de s'aventurer.

   Pourtant, une catgorie de langages, dits ``rflexifs'',
permet d'explorer aussi profondment que l'on veut
cette ``tour'' de mta-langages,
par l'introduction dynamique de nouveaux niveaux ``mta'',
dans le but d'adapter les programmes aux divers contextes
dans lesquels ils sont utiliss sans avoir  les rcrire  chaque fois.

   La gageure est alors de pouvoir exprimer le sens d'un objet
par des mtaobjets lmentaires que l'on combinerait;
mais la mise au point d'une mthode cohrente et simple
pour combiner des mtaobjets reste encore  trouver.
Les trois articles qui sont donns  tudier visent
justement cet objectif.
<p>


<sect1>Les trois articles
<p>

Dans le premier article,
``Metaclass Compatibility'' [Gra89], Nicolas Graube
discute du problme du point de vue des langages  classes et hritage,
o il essaie alors d'tablir des rgles de compatibilit
entre les classes des classes des objets, leurs ``mtaclasses'',
pour assurer la consistence d'un mcanisme d'hritage s'tendant
aux mtaclasses.

Le deuxime article,
``Towards a Methodology for Explicit Composition of MetaObjects'' [MMC95]
de Philippe Mulet, Jacques Malenfant, et Pierre Cointe,
expose un mthodologie pour composer des mtaobjets
dans un langage rflexif avec un protocole  mtaobjets.

Le troisime et dernier article,
``Incremental Redefinition of Open Implementations'' [Mez96],
de Mira Mezini,
propose un modle pour accomplier une redfinition incrmentale des objets
dans les langages  implmentation ouverte.
<p>


<sect>Les articles en situation
<p>

<sect1>Diffrents points de vue sur le mme problme
<p>
Tout en parlant essentiellement du mme sujet,
et tout en suivant une mme tradition de langages dits orients-objets
o l'hritage est le mcanisme privilgi
de dfinition incrmentale des objets,
les auteurs utilisent chacun un vocabulaire qui lui est propre,
li  des nuances spcifiques et  l'affiliation  un courant particulier
de la recherche en la matire.
Par exemple, les mtaobjets  combiner
sont appels ``mtaclasses'' chez Graube,
``mtaobjets'' chez Mulet, Malenfant et Cointe,
et ``mtadescriptions'' chez Mezini;
pourtant,  la lecture des articles,
on discerne peu de diffrence de contenu entre ces termes.

Cette varit de dnomination engendre une certaine confusion chez le lecteur,
et invite  se demander en quoi les lgres diffrences conceptuelles
affectent les argumentations dveloppes.
On se rend alors compte qu'une unit linguistique serait bienvenue,
mais cela ncessiterait un standard reconnu,
fond sur des dfinitions formelles des concepts,
qui n'existe pas actuellement.
<p>



<sect1>Problme des dfinitions
<p>
A dfaut d'un tel standard, le lecteur doit s'en tenir  ce qui
est explicitement dfini dans l'article ou
les ouvrages cits en rfrences bibliographiques  ce propos.
<p>

<sect2>Dfinitions dans [Gra89]
<p>
Des divers auteurs, Nicolas Graube se soucie le plus de
prciser le sens des termes employs:
d'une part, il se place dans le cadre des langages
Smalltalk, ObjVlisp, CLOS,
d'o l'on peut tirer des dfinitions oprationnelles
de classes et mtaclasses,
d'autre part, il tente de donner lui-mme une dfinition
de ces termes, dans la section 2 de son article.

Malheureusement, il sacrifie parfois la prcision
 une volont de gnralit, sans pour autant se dtacher
de particularits des langages  classes.
Par exemple, il dfinit une classe comme
la description de la structure et du comportement de ses instances,
la structure dcrivant elle-mme l'tat propre des instances,
le comportement dcrivant les mthodes s'appliquant aux instances.
Cette dfinition est assez gnrale, donc,
mais elle pourrait tre simplifie
par une approche d'ordre suprieur que CLOS suggre,
unifiant valeurs et fonctions,
et ne laissant comme distinction
que la rsolution de porte des variables entre classe et instance;
une classe ne serait plus que la description
de la structure dynamique des instances.
<p>



<sect2>Dfinitions dans [MMC95]
<p>

Mulet et al. utilisent des termes plus gnraux
au contenu technique plus vague (mtaobjet, composition),
et font appel  des mtaphores qui correspondent mal aux concepts
qu'ils dveloppent ensuite:
ainsi, l'auteur de l'introduction
expose parfaitement en termes synthtiques le problme gnral
de la combinaison de mtaobjets,
mais ds la section 1, l'article devient plus approximatif;
La combinaison de mtaobjets est d'abord assimile
 la composition de fonctions (pour laquelle les auteurs choisissent
une convention inhabituelle sur l'ordre des arguments).
Or, la composition de fonctions consiste  appliquer
l'un aprs l'autre deux objets du premier ordre
avec une contrainte de typage:
<verb>
  f: 'a -> 'b
  g: 'b -> 'c
---------------
g o f: 'a -> 'c
</verb>
Au contraire, la combinaison de mtaobjets consiste
 appliquer <em/en mme temps/
(ce qui est explicitement crit dans l'introduction)
des objets mta, avec une contrainte de typage plutt du genre:
<verb>
  Mb: 'a -> 'b
  Mc: 'a -> 'c
-------------------------
C(Mb,Mc): 'a -> 'C('b,'c)
</verb>
La composition de fonction est encore voque pour modliser l'hritage,
alors que l'hritage fait intervenir deux objets
de nature essentiellement diffrente,
une classe de base et une modification que l'on y <em/applique/;
la composition supposerait que les deux oprandes ainsi que le rsultat
soient des objets fonctionnels de nature similaire.
<p>


<sect2>Dfinitions dans [Mez96]
<p>
L'auteur du troisime article ne donne pas de dfinition formelle,
et les textes auquels il se rfre (notamment [OI94]) non plus;
cependant, de petits exemples et explications informelles
sont donns pour clairer les concepts dvelopps.
L'article apporte donc une certaine intuition du sujet,
mais se prive de la possibilit de prouver un rsultat quelconque,
faute d'un cadre rigoureux de dmonstration.
Les choix architecturaux paraissent en consquence arbitraires,
n'tant pas justifis:
par exemple, la division entre objets et quatre catgories de mtaobjets,
et la sparation correspondante du systme propos en cinq espaces.
Le plus gnant est qu'on ne comprend pas bien
ce que l'auteur met sous les noms de ClassCombiner ou MetaCombiner,
prsent comme la solution au problme;
le pouvoir d'expression et le mcanisme de rsolution de ces
super Combinateurs restent obscurs.
<p>


<sect1>Arguments
<p>

Tous les trois articles partent de la notion d'hritage
comme mcanisme incontournable pour la dfinition d'objets.
Cependant, ils divergent quant au rle de l'hritage dans
la combinaison de mtaobjets.
<p>

<sect2>Argument de [Gra89]
<p>
Pour Graube, c'est videmment par l'hritage que se combinent les mtaclasses:
dans la section 3 de son article, il montre que
si une classe hrite d'une autre,
cela impose immdiatement des contraintes
entre mtaclasse de la nouvelle classe et celle de la classe parent;
la solution de Smalltalk tait que la mtaclasse hrite
de la mtaclasse du parent;
celle de CLOS, que les deux mtaclasses soient gales,
ce qui n'empche pas l'existence de conflits smantiques
entre deux superclasses d'une mme mtaclasse.
Selon Graube dans la section 4 de son article,
plutt que d'imposer une restriction grossire qui ne rsoud rien,
il faut tablir finement les contraintes de compatibilit
requises sur une nouvelle classe et sa mtaclasse
pour pouvoir hriter d'une classe existante.
<p>


<sect2>Argument de [MMC95]
<p>
Pour Mulet et al. les mtaobjets doivent se ``composer'' comme les objets,
par spcialisation (terme qui reprend  la fois l'hritage des
langages  classes et plus gnralement
les mcanismes de dlgation des langages  objets),
et par agrgation (terme dcrivant produits cartsiens et enregistrements),
la diffrence tant dans la propagation du lien dynamique vers
le (mta)objet lors de l'invocation de mthodes.
Pour pouvoir composer des mtaobjets, ils laissent dans l'objet
``compos  gauche'' des ``trous''
remplis par le mtaobjet ``compos  droite''.

Pour viter les conflits entre mtaobjets,
ils proposent comme mthodologie la limitation volontaire
des interactions entre mtaobjets  un protocole  mtaobjet
restreint  deux messages (lookup, apply),
hors desquels les mtaobjets doivent tre des botes noires n'interfrant pas;
un troisime message, compose-with,  la smantique particulire,
sert  composer les mtaobjets.
Aprs avoir illustr l'utilisation de cette mthodologie,
les auteurs montrent comment la transporter dans le cadre de CLOS,
o elle apparat comme une restriction  l'utilisation de mixins
tels que dfinis dans le parent FLAVORS de CLOS.
<p>


<sect2>Argument de [Mez96]
<p>
Dans l'article de Mezini,
la reprsentation d'un objet est disperse
en de multiples ``mtadescriptions'',
o une description par dfaut est distingue de descriptions d'ajustements.
Au cours d'une prsentation des principes d'implmentation ouverte,
sont alors voqus comme spcification informelle
des aspects ``statiques'' et ``dynamiques'' que le systme doit prsenter;
le concept de ``conditions limites dynamiques'' y apparat comme
dterminant pour la validit des mtadescriptions.

Un modle est alors propos pour grer ces descriptions disperses
en tenant compte de leur domaine de validit:
des gestionnaires s'occuperont chacun d'une description,
tandis que des mtaprogrammes propageront dans le programme
des modifications assurant la cohrence des descriptions entre elles.
Les classes sont utilises pour dfinir des descriptions
partielles par dfaut d'un objet, mais la faon dont l'hritage
fait partie de la description par dfaut ou de descriptions d'ajustement
est peu clair.
Chaque objet est alors gr individuellement par son propre
MetaCombinateur qui dispose d'un ensemble dynamique de descriptions
partielles  combiner, ensemble plus tard dcrit comme une liste dynamique
de filtres appliqus dans l'ordre.
<p>


<sect1>Exemples prsents
<p>

Pour illustrer leur propos,
chacun des trois articles prsente des exemples d'application
 l'usage des mtaobjets.

<itemize>
<item>[Gra89]
Graube montre comment utiliser les mtaclasses pour accder
 des donnes concernant la classe entire,
que ce soit une variable partage par les instances (les points colors),
ou la liste des instances de la classe;
il montre aussi comment utiliser les mtaclasses pour
tracer les modifications apportes  un objet,
ou pour spcifier leur reprsentation concrte.
Il illustre honntement l'usage des mtaclasses,
et fait comprendre les contraintes lies  leur combinaison.

<item>[MMC95]
Mulet et al. citent des mtaobjets
pour dfinir la smantique de base d'un objet,
grer le mcanisme de dlgation,
effectuer des oprations d'aide  la visualisation et au dbogage,
forcer ou vrifier le respect de contraintes.
Ils n'illustrent cependant pas l'usage de leur mthodologie
dans un exemple concret, prfrant en suggrer l'ide gnrale
dans des diagrammes.

<item>[Mez96]
Le seul exemple de Mezini est celui d'objets mutables lignes
ayant des contraintes bizarres sur la faon de les modifier,
qui doivent tre propages sur la faon dont ragissent les
objets polygnes mutables les utilisant.
Ces contraintes et constructions,
qui semblent arbitrairement nonces et sont assez dconcertantes,
servent sans doute  dmontrer la gnralit du mcanisme propos.
La smantique des objets obtenus est vraiment trs complexe,
les objets ayant une interface imprative dans le plus pur style C++,
plutt qu'une interface fonctionnelle.
</itemize>
<p>


<sect>Discussion
<p>

<sect1>Points communs et divergences
<p>

<sect2>Un Schma base+modifications
<p>
Dans les trois articles,
un objet est d'abord dcrit par une classe,
(``classe'' dans [Gra89],
``mtaobjet de base'' dans [MMC95],
``classe'' ou ``description par dfaut'' dans [Mez96]);
des mtaobjets ``modifient'' alors cette description de base
(dans [Gra89], des ``mtaclasses'',
dans [MMC95], des ``mtaobjets composables  gauche''
dans [Mez96], des ``descriptions d'ajustement'').

La rpartition des bases et modifications entre mtaniveaux
se fait diffremment dans les articles:
<itemize>
<item>
On remarque que dans [Gra89],
ces modifications sont compltement transfres au niveau mta,
tout en restant somme toute modestes.
Rien ne semble empcher les mtaclasses d'avoir elles-mmes des classes,
au contraire, le systme tant alors (au moins partiellement) rflexif.
<item>
Dans [MMC95], on ne voit qu'un seul niveau, rflexif,
d'objets et de mtaobjets, et les mtaobjets de modifications
(mtaobjets avec trous ou mixins) sont au mme niveau
que les mtaobjets de base (mtaobjets sans trou ou classes).
<item>
Dans [Mez96], les modifications sont effectues par des
descriptions d'ajustement,
de mme niveau que les descriptions par dfaut,
mais compartimentes dans un autre ``espace''.
Mais les mtaobjets ne sont pas ouvertement considrs eux-mmes
comme des objets, ce qui vite de rpter rcursivement
le schma dj assez compliqu de mtaobjets en prsence.
</itemize>

Cette diffrence de rpartition en soi ne semble
qu'une diffrence arbitraire de vocabulaire,
et tmoigne d'une absence de thorie claire
en ce qui concerne la rflexivit et l'embotement des mta-niveaux,
ce qui rend floue l'attribution de chaque caractristique  un niveau
de mtalangage.
<p>


<sect2>Responsabilit de la cohrence
<p>
   De manire plus importante, Graube tente de dfinir des rgles
formelles que le systme pourrait vrifier lui-mme,
mais n'en expose que le principe gnral,
tant finalement bloqu par les complications smantiques
ncessits par la dclaration et la propagation des hypothses
sur les mtaclasses.

Mulet et al., en comparaison,
laissent au programmeur la charge
complte d'assurer la cohrence du systme,
ce pour quoi ils ne donnent pas d'indice dans leur mthodologie;
c'est mme au programmeur de grer la composition des mtaobjets
en dfinissant la rponse au message <tt/compose-with/!
Il apparat alors que [MMC95] dfinit
une interface de bas niveau  la combinaison de mta-objets,
qui ne fait que repousser le problme sans le rsoudre:
les auteurs interdisent  la non-orthogonalit des mta-objets d'apparatre
dans leur interface mta, mais ceux-ci sont toujours libres d'interfrer
arbitrairement dans le niveau de base, sans qu'une solution ne soit propose.

Enfin, Mezini semble demander de l'utilisateur qu'il spcifie lui-mme,
dans des ``mtaprogrammes'', comment assurer la cohrence du systme,
autre faon de repousser le problme sans le rsoudre;
la mthode de Mezini a au moins l'avantage de prvoir une mta-interface
qui permette cette rsolution.
<p>


<sect1>Noeud du problme
<p>

<sect2>Les articles ne rsolvent pas le problme
<p>
[Gra89] est le seul article  vraiment tenter de donner un sens
 la combinaison simultane de mtaclasses, mais il n'aboutit pas;
[MMC95] et [Mez96], tout en reconnaissant les problmes soulevs,
les fuient en proposant une composition srielle de filtres,
sans pouvoir s'assurer que ceux-ci n'interfrent pas entre eux.

Dans les trois cas, des prjugs informels
ont remplac une formalisation explicite.
<p>

<sect2>Hritage contre sous-typage
<p>
Les rgles que Nicolas Graube souhaite voir propages par ses objets
sont typiquement des rgles de <em/sous-typage/, et non pas d'hritage:
un sous-type serait assur de conserver toutes les hypothses
sur les instances du type originel,
tandis qu'un type hrit non-trivial est assur d'en modifier certaines;
Graube trbuche sur cette difficult,
et avoue en fin d'article qu'il n'arrive pas, dans le cadre de l'hritage,
 trouver o dclarer les contraintes,
et que l'analyse smantique du systme obtenu est un travail trs ardu;
la seule application qu'il arrive  montrer consiste  spcifier
la restriction abusive de la formation de classes  l'hritage simple.

Dans l'article [EST95], une quipe de recherche est arrive
 donner une smantique formelle de l'hritage
en passant par le sous-typage et les types polymorphes paramtriques;
le contraire semble impossible,
l'hritage tant  la fois plus compliqu et moins expressif
que la combinaison des deux concepts prcdents.

Nous pensons qu'abandonner l'hritage et utiliser le sous-typage
auraient grandement simplifi le problme
et permis d'aboutir  une algbre de types assez simple
pour tre dcrite dans un article
et implmente dans un langage.
Cela ne suffirait certes pas  rsoudre le problme de combinaison,
mais l'aurait dbarass des obstacles qui ont arrt Graube.
<p>


<sect2>``composition'' contre ordre suprieur
<p>
Mulet et al. confondent visiblement composition et application,
essayant d'implmenter l'un par l'autre:
ils dcrivent en terme de composition l'application de mixins,
et proposent de raliser la composition par l'envoi
d'un message de composition  l'un des mtaobjets.
La composition d'objets implique naturellement
que ces objets soient des fonctions,
et les auteurs introduisent eux-mmes la notion de ``trou''
dans une dfinition,
mais enferms dans le formalisme du passage de messages,
ils ne voient pas que l'introduction explicite
de mtaobjets fonctionnels d'ordre suprieur ou foncteurs,
trivialiseraient leurs problmes de mise en place d'une composition.

Mulet et al. prnent l'orthogonalit des concepts de programmation,
mais font appel aux notions visiblement voisines
de spcialisation et agrgation,
alors qu'ici encore, des constructeurs d'ordre suprieur
pour des types paramtriques et points fixes rendraient
orthogonaux agrgation de types et rsolution de type  passer en paramtre.
Les foncteurs et modules d'ordre suprieur ont t dcrits,
entre autre, dans [Blu94] et [Ler95].

Encore une fois, une telle simplification ne suffirait pas  rsoudre
le problme de combinaison des mtaobjets,
mais aurait au moins permis de formaliser proprement
les mcanismes utiliss,
pour pouvoir discuter de leur adquation au problme trait.
<p>


<sect2>Cloisonnement arbitraire contre sparation smantique
<p>
Mezini reprend  d'autres auteurs des problmatiques
et des propositions intressantes [IB+94] [OI94],
mais en fait un mlange plutt qu'une synthse.
L'article prtend trouver une solution pour combiner des
mtaobjets, mais le peu qu'il ressort des explications floues de l'auteur
laisse le lecteur dubitatif.
L'valuation de l'approche propose dans la section 5 est
abusivement optimiste:
<itemize>
<item>le cloisonnement des descriptions dans des espaces arbitraires
	ne saurait suffire  rendre la sparation propre;
	la complexit des interactions
	est dplace vers gestionnaires et mtaprogrammes,
	et non point rsolue;
<item>il semble que les extensions incrmentales doivent prendre
	explicitement en considration toutes les extensions prcdentes
	dans un mtaprogramme pour pouvoir se combiner avec elles;
<item>dans ces conditions, le contrle de porte n'est videmment
	pas assur,  moins de restreindre les ajustements
	que peuvent ou doivent faire les mtaprogrammes;
<item>le systme ne procure aucune interoprabilit,
	et c'est au programmeur de s'assurer que ses ajustements
	ne casseront pas un programme prcdent.
<item>rutiliser des descriptions similaires dans des contextes similaires
	ressemble fort  du clone-and-hack,
	ce qui n'a rien d'une bonne rutilisabilit.
<item>l'extensibilit des programmes ne fait pas de doute;
	mais qu'en est-il des mta-programmes?
	Y a-t-il une architecture rflexive (qui n'est pas voque),
	ou les mtaprogrammes doivent-ils tre compltement rcrits
	pour reflter les extensions des programmes?
</itemize>
Les notes finales sur l'tat d'une implmentation du modle
ne sont pas encourageantes.


<sect>Conclusions
<p>

<sect1>Comparaison avec d'autres travaux
<p>

<sect2>Travaux similaires
<p>
L'article de Chiba sur OpenC++ v2 [Chi95] est plus clairant
sur les applications pratiques et concrtes des mtaobjets;
 dfaut de thorie, Chiba y offre le fruit d'une longue
exprience sur des situations relles;
il ne s'attarde pas sur les problmes de combinaison de mtaobjets,
mais montre qu'il a su les traiter naturellement,
puisque sans utiliser d'artifices compliqus et arbitraires,
il permet aussi bien le contrle de l'hritage des mtaclasses,
et la composition squentielle de mtaclasses multiples,
lesquels sont aussi les deux propositions
que l'on peut dduire des trois articles tudis.
<p>

<sect2>Partir de bases solides
<p>
Les trois articles se sont lancs dans le problme de
la combinaison de mtaobjets sans en avoir cern les enjeux
avec une prcision suffisante.
Si les mtaobjets sont cens apporter des modifications smantiques
thoriques ou pratiques, de quelles sortes de modifications s'agit-il,
et sur quel substrat smantique?
Quels effets s'agit-il d'obtenir?

On ne saurait se contenter de dire qu'il y a modification,
car quand on modifie un objet, on n'a par principe plus le mme objet,
et on ne peut donc a priori rien en dire,
 moins d'avoir une thorie prcise des modifications possibles.
<p>

<sect1>Opinion personnelle
<p>

<sect2>Une voie  explorer
<p>
Nous pensons qu'une voie prometteuse est de considrer
une opration de reprsentation concrte comme la rciproque
de l'interprtation abstraite;
la rflexivit servirait alors  <em/tendre/ la smantique
des objets existants vers des domaines plus concrets
de persistence, interface machine, interface utilisateur, etc;
L'extension vers des smantiques concrtes
permet aussi de dfinir de nouveaux objets prcdemment inexprimables.

La contrainte  respecter pour tre une extension
serait qu'il existe une interprtation abstraite
transformant tout objet tendu en l'objet exact de dpart,
conservant donc toute l'information abstraite.
En comparaison, les mthodes actuelles ne garantissent
aucune conservation d'information par rflection.
La contrainte de compatibilit
entre deux reprsentations R1 et R2 d'une structure S,
rciproques respectivement des interpretations I1 et I2,
serait qu'il existe une reprsentation R3 pouvant les unifier,
en ce que l'interprtation rciproque I3 de R3 puisse factoriser
aussi bien  travers I1 et I2.

Bien sr, ce ne sont l que des spculations  explorer plus avant.
<p>

<sect2>Le mot de la fin
<p>
La rflexivit est dj un sujet assez difficile.
Plutt que de se compliquer la vie et de tenter de la greffer
sur des systmes  la smantique confuse
dont la conception tient du bricolage,
comme les systmes  hritage,
les chercheurs devraient plutt partir de systmes
ayant des smantiques formelles les plus simples possibles
(langages fonctionnels, langages  acteurs,
et autres variations sur lambda- et pi- calculs formels);
ils pourraient alors clairement comprendre et thoriser
en quoi la rflexivit tend ces systmes.
<p>


<sect>APPENDICES
<p>

<sect1>Bibliographie
<p>
<itemize>
<item>[Blu94]
	Matthias Blume.
	Separate Compilation for Scheme.
	1994
<item>[Chi95]
	S. Chiba.
	A Metaobject Protocol for C++.
	In ACM, editor, OOPSLA, pages 285-299,
	1995
<item>[EST95]
	J. Eifrig, S. Smith, V. Trifonov.
	Sound Polymorphic Type Inference for Objects.
	OOPSLA (?),
	1995
<item>[Gra89]
	Nicolas Graube.
	Metaclass Compatibility.
	ACM SIGPLAN Notices, editor, OOPSLA, pages 305-315,
	1989
<item>[IB+94]
	Jon Inouye, Andrew Black, Calton Pu, et Jonathan Walpole.
	Domesticating Dynamic Diversity.
	OGI,
	1994
<item>[Ler95]
	Xavier Leroy.
	Applicative functors for fully transparent higher-order modules.
	POPL,
	1995
<item>[Mez96]
	Mira Mezini.
	Incremental Redefinition of Open Implementations.
	Ch. Zimmermann ed. Advances in Metalaevel Architectures and Reflection,
	1996
<item>[MMC95]
	Philipple Mulet, Jacques Malenfant, and Pierre Cointe.
	Towards a Methodology for Explicit Composition of MetaObjects.
	ACM SIGPLAN Notices, 30(10):316-330,
	October 1995
<item>[OI94]
	Open Implementation Workshop.
	<url url="http://http://www.parc.xerox.com/spl/projects/oi/oi-workshop-94">,
	1994
</itemize>


</article>
END
;

s/\[/\&lsqb\;/mg ;
s/\]/\&rsqb\;/mg ;

open FOO, ">combiner-meta.fr.sgml" ;
print FOO $_ ;
